/*{{{}}}*/
/*{{{  COMMENT*/
/*bmp4ptl - Create bitmap of the planar transmission line.*/
/*(c) 2023, Takayuki HOSODA*/
/*http://www.finetune.co.jp/~lyuka/technote/cbcpw/*/
/*}}}  */
/*{{{  #include*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <float.h>
#include <stdint.h>
#include <sys/types.h>
/*}}}  */
/*{{{  #define constants*/
#define NO_ERR           0
#define E_HELP           1
#define E_VERSION        2
#define E_OPTION         3
#define E_FILE           4
#define E_FOPEN          5
#define E_FWRITE         6
#define E_MALLOC         7
#define E_WIDTH          8
#define E_SPACING        9
#define E_HEIGHT        10
#define E_THICKNESS     11
#define E_DEPTH         12
#define E_GAP           13
#define E_COATING       14
#define E_COATSPACE     15
#define E_PERMITIVITY   16
#define E_NARROW        17
#define E_BOX_SIZE      18
#define E_BOX_NARROW    19
#define E_BOX_LOW       20
#define E_SMALL_DIM     21

#define F_VERBOSE        1
#define F_QUIET          2
#define F_ARRAY          4
#define F_CSV            8
#define F_PTIME         16
#define F_FILL          32
#define F_STRIP         64
#define F_COUPLER      128
#define F_WIRE         256
#define F_HINOMARU     512
#define F_ATLC        1024
#define F_DEBUG       2048

#define MAX_SIZE     32767
#define RIM_SIZE         5
#define GRID_MIN        25

#define C_GND            0
#define C_POS            1
#define C_NEG            2
#define C_AIR            3
#define C_SUB            4
#define C_FIL            5
#define C_INS            6
#define C_MAX            6
#define V_GREEN   0x00ff00
#define V_RED     0xff0000
#define V_BLUE    0x0000ff
#define V_WHITE   0xffffff
#define V_YELLOW  0xffff00
#define V_CYAN    0x00ffff
#define V_PINK    0xffaaff
/*}}}  */
/*{{{  struct*/
struct DIB_HEADER {
    int16_t bfType;
    int32_t bfSize;
    int16_t bfReserved1;
    int16_t bfReserved2;
    int32_t bfOffBits;
    int32_t biSize;
    int32_t biWidth;
    int32_t biHight;
    int16_t biPlanes;
    int16_t biBitCount;
    int32_t biCompression;
    int32_t biSizeImage;
    int32_t biXpelsPerMeter;
    int32_t biYpelsPerMeter;
    int32_t biClrUsed;
    int32_t biClrImportant;
};
struct          DIB_HEADER dib;
/*}}}  */
/*{{{  global variables*/
/*{{{  property*/
char *myname = "bmp4ptl";
char *help = "Creates a bitmap of cross-section of a planar transmission line.";
char *version = "Rev. 0.75 (Jan. 21, 2024)";
char *copyright = "(c) 2023, Takayuki HOSODA";
char *site = "http://www.finetune.co.jp/~lyuka/technote/bmp4ptl/";
/*}}}  */
/*{{{  usage*/
char *usage = "\
SYNOPSIS\n\
  bmp4ptl [OPTIONS] FILENAME\n\
OPTIONS\n\
  -array   : print properties in array form.\n\
  -atlc    : print command line for atlc.\n\
  -c=nn    : coating thickness of the wire\n\
  -CSV     : print properties in comma separated variables\n\
  -d=nn    : burying depth\n\
  -ec=f.f  : permitivity of the coating (dafult : = er)\n\
  -ef=f.f  : permitivity of the filler (dafult : = er)\n\
  -er=f.f  : permitivity of the substrait (dafult : 4.7)\n\
  -fill    : fill to the upper ground conductor\n\
  -g=nn    : coupling gap\n\
  -h=nn    : height of the dielectric\n\
  -H=nn    : box height (default : 8 * h + t)\n\
  -help    : show this help\n\
  -hinomaru: Draw the national flag of Japan (Hinomaru)\n\
  -ptime   : print timestamp\n\
  -quiet   : quiet (suppress information output)\n\
  -s=nn    : spacing to the side ground condutors\n\
  -strip   : strip all ground conductors on both side\n\
  -t=nn    : thickness of the conductor\n\
  -verbose : print verbose informations\n\
  -w=nn    : width of the center conductor strip\n\
  -wire    : use circular wire as the signal condcutor\n\
  -W=nnn   : box width (default : 12 * (h + t) + g + 2 * s + 2 * w\n\
             for couplers, otherwise 12 * h + 4 * t + 2 * s + w)\n\
FILENAME\n\
  output filename to write bitmap.\n\
EXAMPLE\n\
  bmp4ptl -w=200 -h=200 -s=200 -t=18 -g=100 -er=4.6 out.bmp\n\
  bmp4ptl -w=280 -h=200 -s=200 -t=18 -d=50 -er=3.4 -ef=4.6 out.bmp\n\
  bmp4ptl -w=230 -h=200 -strip -c=16 -er=4.5 -ec=5.0 -wire out.bmp\n\
STRUCTURE\n\
  Single ended transmission line                                     \n\
   |<-------------------------W--------------------------|           \n\
  ========================================================= -------  \n\
  =  vaccum                                               =       |  \n\
  =               |<--s-->|<--w-->|<--s-->|               =       |  \n\
  =               |       |       |       |               =       |  \n\
  =@@ef@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@= ---b  |  \n\
  ================@@@@@@@@@+++++++@@@@@@@@@================ ---t  | H\n\
  =#######################################################=  |    |  \n\
  =# er ##################################################=  | h  |  \n\
  =#######################################################=  |    |  \n\
  ========================================================= -------  \n\
                                          Length unit is in [pixel]  \n\
  Coupler                                                            \n\
   |<-------------------------W--------------------------|           \n\
  ========================================================= -------  \n\
  =  vaccum                                               =       |  \n\
  =       |<--s-->|<--w-->|<--g-->|<--w-->|<--s-->|       =       |  \n\
  =       |       |       |       |       |       |       =       |  \n\
  =@@ef@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@= ---b  |  \n\
  ========@@@@@@@@@+++++++@@@@@@@@@-------@@@@@@@@@======== ---t  | H\n\
  =#######################################################=  |    |  \n\
  =# er ##################################################=  | h  |  \n\
  =#######################################################=  |    |  \n\
  ========================================================= -------  \n\
  = denotes ground conductors, + - denotes signal conductors,\n\
  # denotes the substrait and @ denotes filler (or top-coat).\n\
COLORS\n\
  pure Red        (#FF0000) : +Signal conductor\n\
  pure Blue       (#0000FF) : -Signal conductor\n\
  pure Green      (#00FF00) : Ground conductor\n\
  Yellow          (#FFFF00) : Substrate dielectric\n\
  Cyan            (#00FFFF) : Filler dielectric\n\
  Pink            (#FFAAFF) : Wire insulation dielectric\n\
  White           (#FFFFFF) : Vaccum\n\
BUGS\n\
  This program is a front-end processor for 2D field solvers, so it's not very\n\
  useful by itself.\n\
  Wires with oval cross sections are not supported.\n\
  Simple grids are not suitable for representing circular cross sections.\n\
";
/*}}}  */
/*}}}  */
/*{{{  errmsg(err)*/
int
errmsg(err)
    int err;
{
    switch(err){
    case NO_ERR:
        break;
    case E_HELP:
        fprintf(stdout, "%s: %s \n%s %s\n  %s\n%s", myname, help, version, copyright, site, usage);
        break;
    case E_VERSION:
        fprintf(stdout, "%s %s\n", myname, version);
        break;
    case E_OPTION:
        fprintf(stderr, "%s: Incorrect option.\n", myname);
        break;
    case E_FILE:
        fprintf(stderr, "%s: Output file name must be specified.\n", myname);
        break;
    case E_FOPEN:
        fprintf(stderr, "%s: Cannot open output file.\n", myname);
        break;
    case E_FWRITE:
        fprintf(stderr, "%s: An error was encountered while writing the output file.\n", myname);
        break;
    case E_MALLOC:
        fprintf(stderr, "%s: Could not allocate memory.\n", myname);
        break;
    case E_WIDTH:
        fprintf(stderr, "%s: Width w must be breater or equal to 1 (1 <= w).\n", myname);
        break;
    case E_SPACING:
        fprintf(stderr, "%s: Spacing s must be greater or equal to 1 (1 <= s).\n", myname);
        break;
    case E_HEIGHT:
        fprintf(stderr, "%s: Height h must be greater or equal to 1 (1 <= h).\n", myname);
        break;
    case E_THICKNESS:
        fprintf(stderr, "%s: Thickness t must be greater or equal to 1 (1 <= t).\n", myname);
        break;
    case E_COATING:
        fprintf(stderr, "%s: Coating thickness c must be greater or equal to 0 (0 <= c).\n", myname);
        break;
    case E_COATSPACE:
        fprintf(stderr, "%s: Spacing s must be greater or equal to the coating thickness c (c <= s).\n", myname);
        break;
    case E_DEPTH:
        fprintf(stderr, "%s: Depth d must be greater or equal to 0 (0 <= d).\n", myname);
        break;
    case E_GAP:
        fprintf(stderr, "%s: Gap g must be greater or equal to 1 (1 <= g).\n", myname);
        break;
    case E_NARROW:
        fprintf(stderr, "%s: Gap s must be greater or equal to 1 (1 <= s).\n", myname);
        break;
    case E_PERMITIVITY:
        fprintf(stderr, "%s: Permitivity er must be greater or equal to 1.0 (1 <= er).\n", myname);
        break;
    case E_BOX_SIZE:
        fprintf(stderr, "%s: Box size too large (H | W < %d).\n", myname, (MAX_SIZE) - 2 * (RIM_SIZE));
        break;
    case E_BOX_NARROW:
        fprintf(stderr, "%s: Box width W must be wider than w + 2s.\n", myname);
        break;
    case E_BOX_LOW:
        fprintf(stderr, "%s: Box height H must be higher than h + d and h + b.\n", myname);
        break;
    case E_SMALL_DIM:
        fprintf(stderr, "%s: Warning. There are dimensions with a minimum grid count of 25 or less.\n", myname);
        break;
    default:
        fprintf(stderr, "%s: Unknown error occured.\n", myname);
        break;
    }
    return(err);
}
/*}}}  */
/*{{{  round(x)*/
#if HAVE_C99_ROUND
double round(double x);
#else
double
round(double x) {
    if ( x > 0.0 ) {
        return floor(x + 0.5);
            } else {
        return -1.0 * floor(fabs(x) + 0.5);
    }
}
#endif
/*}}}  */
/*{{{  hypot(x, y)*/
double
hypot(double x, double y) {
    double a, b;
    x = fabs(x);
    y = fabs(y);
    if(x == 0) return(y);
    if(y == 0) return(x);
    if(y > x) {
        a = x; x = y; y = a;
    }
    a = x - y;
    if(a > y){
        a = x / y;
        a += sqrt(1.0 + a * a);
    } else {
        a /= y;
        b = a * (a + 2.0);
        a += b / (M_SQRT2 + sqrt(2.0 + b));
        a += M_SQRT2 + 1.0;
    }
    return(x + y / a);
}
/*}}}  */
/*{{{  gcd(a, b)*/
int32_t
gcd(a, b)
    int32_t         a;
    int32_t         b;
    {
        int32_t     t;
        do {
            if (a <= b) {
                t = a;
                a = b;
                b = t;
            }
            a %= b;
        } while (a);
    return (b);
}
/*}}}  */
/*{{{  max(a, b)*/
int32_t
max(int32_t a, int32_t b) {
    return a > b ? a : b;
}
/*}}}  */
/*{{{  min(a, b)*/
int32_t
min(int32_t a, int32_t b) {
    return a <= b ? a : b;
}
/*}}}  */
/*{{{  sqr(a)*/
int32_t
sqr(int32_t a) {
    return a * a;
}
/*}}}  */
/*{{{  parse(argc, argv, flags, filename, w, s, h, t, d, g, c, er, ef, ec, H, W)*/
int
parse(argc, argv, flags, filename, w, s, h, t, d, g, c, er, ef, ec, H, W)
    /*{{{  variables*/
    int             argc;
    char           *argv[];
    int            *flags;
    char          **filename;
    double         *w;
    double         *s;
    double         *h;
    double         *t;
    double         *d;
    double         *g;
    double         *c;
    double         *er;
    double         *ef;
    double         *ec;
    double         *H;
    double         *W;
    /*}}}  */
{
    /*{{{  variables*/
    int             i;
    int          rtnv;
    int         filec;
    /*}}}  */
    /*{{{  initialise*/
    *filename =  NULL;
    filec     =     0;
    *flags    =     0;
    rtnv      =     0;
    *w        =   100.0;
    *s        =   100.0;
    *h        =   100.0;
    *t        =     0.0;
    *d        =     0.0;
    *g        =     0.0;
    *c        =     0.0;
    *er       =     1.0;
    *ef       =     0.0;
    *ec       =     0.0;
    *H        =     0.0;
    *W        =     0.0;
    /*}}}  */
    if (argc == 1) return (rtnv = E_HELP);
    /*{{{  get options*/
    for (i = 1; i < argc; i++) {
        if (*argv[i] == '-') {
            /*{{{  options*/
            switch (*++argv[i]) {
            /*{{{  array / atlc*/
            case 'a':
                if (!strncmp(argv[i], "array", strlen("array"))) {
                   *flags |= F_ARRAY;
                } else if (!strncmp(argv[i], "atlc", strlen("atlc"))) {
                   *flags |= F_ATLC;
                }
                break;
            /*}}}  */
            /*{{{  box Width*/
            case 'W':
                if (!strncmp(argv[i], "W=", strlen("W="))) {
                    sscanf(&argv[i][strlen("W=")], "%lg", W);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  box Height*/
            case 'H':
                if (!strncmp(argv[i], "H=", strlen("H="))) {
                    sscanf(&argv[i][strlen("H=")], "%lg", H);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  coating thickness*/
            case 'c':
                if (!strncmp(argv[i], "c=", strlen("c="))) {
                    sscanf(&argv[i][strlen("c=")], "%lg", c);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  CSV*/
            case 'C':
                if (!strncmp(argv[i], "CSV", strlen("CSV"))) {
                   *flags |= F_CSV;
                }
                break;
            /*}}}  */
            /*{{{  depth*/
            case 'd':
                if (!strncmp(argv[i], "d=", strlen("d="))) {
                    sscanf(&argv[i][strlen("d=")], "%lg", d);
                } else if (!strncmp(argv[i], "depth=", strlen("depth="))) {
                    sscanf(&argv[i][strlen("depth=")], "%lg", d);
                } else if (!strncmp(argv[i], "d=fill", strlen("d=fill"))) {
                   *flags |= F_FILL;
                   *d = 0;
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  DEBUG*/
            case 'D':
                if (!strncmp(argv[i], "DEBUG", strlen("DEBUG"))) {
                   *flags |= F_DEBUG;
                }
                break;
            /*}}}  */
            /*{{{  fill*/
            case 'f':
                if (!strncmp(argv[i], "fill", strlen("fill"))) {
                   *flags |= F_FILL;
                   *d = 0;
                }
                break;
            /*}}}  */
            /*{{{  gap*/
            case 'g':
                if (!strncmp(argv[i], "gap=", strlen("gap="))) {
                    sscanf(&argv[i][strlen("gap=")], "%lg", g);
                } else if (!strncmp(argv[i], "g=", strlen("g="))) {
                    sscanf(&argv[i][strlen("g=")], "%lg", g);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  height / help / hinomaru*/
            case 'h':
                if (!strncmp(argv[i], "height=", strlen("height="))) {
                    sscanf(&argv[i][strlen("height=")], "%lg", h);
                } else if (!strncmp(argv[i], "h=", strlen("h="))) {
                    sscanf(&argv[i][strlen("h=")], "%lg", h);
                } else if (!strncmp(argv[i], "help", strlen("help"))) {
                    rtnv = E_HELP;
                } else if (!strncmp(argv[i], "hinomaru", strlen("hinomaru"))) {
                    *flags |= (F_HINOMARU | F_STRIP);
                    *s = 0;
                } else if (1 == strlen(argv[i])) {
                    rtnv = E_HELP;
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  print timestamp*/
            case 'p':
                if (!strncmp(argv[i], "ptime", strlen("ptime"))) {
                   *flags |= F_PTIME;
                }
                break;
            /*}}}  */
            /*{{{  thickness*/
            case 't':
                if (!strncmp(argv[i], "thickness=", strlen("thickness="))) {
                    sscanf(&argv[i][strlen("thickness=")], "%lg", t);
                } else if (!strncmp(argv[i], "t=", strlen("t="))) {
                    sscanf(&argv[i][strlen("t=")], "%lg", t);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  quiet*/
            case 'q':
                *flags |= F_QUIET;
                break;
            /*}}}  */
            /*{{{  relative permitivity*/
            case 'e':
                if (!strncmp(argv[i], "er=", strlen("er="))) {
                    sscanf(&argv[i][strlen("er=")], "%lg", er);
                } else if (!strncmp(argv[i], "ef=", strlen("ef="))) {
                    sscanf(&argv[i][strlen("ef=")], "%lg", ef);
                } else if (!strncmp(argv[i], "ec=", strlen("ec="))) {
                    sscanf(&argv[i][strlen("ec=")], "%lg", ec);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  spacing go side ground / strip*/
            case 's':
                if (!strncmp(argv[i], "strip", strlen("strip"))) {
                    *flags |= F_STRIP;
                    *s = 0;
                } else if (!strncmp(argv[i], "s=strip", strlen("s=strip"))) {
                    *flags |= F_STRIP;
                    *s = 0;
                } else if (!strncmp(argv[i], "slot=", strlen("slot="))) {
                    sscanf(&argv[i][strlen("slot=")], "%lg", s);
                } else if (!strncmp(argv[i], "s=", strlen("s="))) {
                    sscanf(&argv[i][strlen("s=")], "%lg", s);
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  verbose / version*/
            case 'v':
                if (!strncmp(argv[i], "version", strlen("version"))) {
                    rtnv = E_VERSION;
                } else if (!strncmp(argv[i], "verbose", strlen("verbose"))) {
                    *flags |= F_VERBOSE;
                } else if (1 == strlen(argv[i])) {
                    *flags |= F_VERBOSE;
                }
                break;
            /*}}}  */
            /*{{{  width / wire*/
            case 'w':
                if (!strncmp(argv[i], "width=", strlen("width="))) {
                    sscanf(&argv[i][strlen("width=")], "%lg", w);
                } else if (!strncmp(argv[i], "w=", strlen("w="))) {
                    sscanf(&argv[i][strlen("w=")], "%lg", w);
                } else if (!strncmp(argv[i], "wire", strlen("wire"))) {
                    *flags |= F_WIRE;
                } else {
                    rtnv = E_OPTION;
                }
                break;
            /*}}}  */
            /*{{{  default*/
            default:
                rtnv = E_OPTION;
                break;
            /*}}}  */
            }
            --argv[i];
            /*}}}  */
        }else{
            filec = i;
        }
    }
    /*}}}  */
    /*{{{  check properties*/
    if (!((rtnv == E_HELP) || (rtnv == E_VERSION))) {
        /*{{{  check width*/
        if (*w < 1.0) {
            rtnv = E_WIDTH;
        }
        /*}}}  */
        /*{{{  check height*/
        if (*h < 1.0) {
            rtnv = E_HEIGHT;
        }
        /*}}}  */
        /*{{{  check slot (side gap)*/
        if ((*s < 1.0) && (!(*flags & F_STRIP))){
            rtnv = E_SPACING;
        }
        /*}}}  */
        /*{{{  check thickness*/
        if (*t == 0) {
            if (*flags & F_WIRE) {
                *t = *w;
            } else {
                *t = 10.0;
            }
        } else if (*t < 1.0) {
            rtnv = E_THICKNESS;
        }
        /*}}}  */
        /*{{{  check depth*/
        if (*d < 0) {
            rtnv = E_DEPTH;
        }
        /*}}}  */
        /*{{{  check gap*/
        if (*g < 0) {
            rtnv = E_GAP;
        } else if (*g > 0) {
            *flags |= F_COUPLER;
        }
        /*}}}  */
        /*{{{  check coating*/
        if (*c < 0) {
            rtnv = E_COATING;
        } else if ((*s < *c) && (*flags & F_WIRE) && (!(*flags & F_STRIP))) {
            rtnv = E_COATSPACE;
        }
        /*}}}  */
        /*{{{  check H*/
        /*{{{  set default value*/
        if (*H <= 0 ) {
            *H = 8.0 * (*h) + ((*flags & F_WIRE) ? ((*w) + (*c)) : (*t));
        }
        /*}}}  */
        /*{{{  check maximum size limited by MAX_SIZE*/
        if ((*H >= (MAX_SIZE - 2.0 * RIM_SIZE)) || (*W >= (MAX_SIZE - 2.0 * RIM_SIZE))) {
            rtnv = E_BOX_SIZE;
        }
        /*}}}  */
        if (*flags & F_WIRE) {
            /*{{{  height limit by wire size*/
            if ((*H < (*h) + (*d)) || (*H <= (*h) + (*w) + 2 * (*c))) {
                rtnv = E_BOX_LOW;
            }
            /*}}}  */
        } else {
            /*{{{  height limit by conductor thickness*/
            if ((*H < (*h) + (*d)) || (*H <= (*h) + (*t))) {
                rtnv = E_BOX_LOW;
            }
            /*}}}  */
        }
        if (*flags & F_FILL) {
            *d = (*H) - (*h);
        }
        /*}}}  */
        /*{{{  check W*/
        if (*W <= 0 ) {
            if (*flags & F_COUPLER) {
                /*{{{  set dafault value for a coupler*/
                *W = 12.0 * (*h) + 2.0 * (*s) + 2.0 * (*w) + (*g);
                /*}}}  */
            } else {
                /*{{{  set dafault value a single ended line*/
                *W = 12.0 * (*h) + 4 * (*t) + 2.0 * (*s) + (*w);
                /*}}}  */
            }
        } else if (*W < (*s + *w + *s)) {
            rtnv = E_BOX_NARROW;
        }
        /*}}}  */
        /*{{{  check er, ef, ec*/
        if (*er < 1.0){
            rtnv = E_PERMITIVITY;
        } else if (*ef == 0.0 ) {
            *ef = *er;
        } else if (*ec == 0.0 ) {
            *ec = *er;
        } else if ((*ef < 1.0) || (*ec < 1.0)) {
            rtnv = E_PERMITIVITY;
        }
        /*}}}  */
        /*{{{  check filename*/
        if (filec == 0) {
            rtnv = E_FILE;
        } else if ((rtnv != E_HELP) && (rtnv != E_VERSION)) {
            *filename = argv[filec];
        }
        /*}}}  */
    }
    
    /*}}}  */
    return (rtnv);
}
/*}}}  */
/*{{{  init_dib(x, y)*/
void
init_dib(x_size, y_size)
    int32_t x_size;
    int32_t y_size;
{
    dib.bfType = 0x4d42;
    dib.bfReserved1 = 0;          /* always 0 */
    dib.bfReserved2 = 0;          /* always 0 */
    dib.bfOffBits = (int32_t) 54; /* without palette */
    dib.biSize = 0x28;
    dib.biWidth = x_size;
    dib.biHight = y_size;
    dib.biPlanes = 1;             /* always 1 */
    dib.biBitCount = 24;          /* 24-bit color */
    dib.biCompression = 0;
    dib.biSizeImage = dib.biHight * (3 * dib.biWidth + (4 - (3 * dib.biWidth) % 4) % 4);
    dib.biXpelsPerMeter = 0;
    dib.biYpelsPerMeter = 0;
    dib.biClrUsed = 0;            /* 0 for all */
    dib.biClrImportant = 0;       /* 0 for all */
    dib.bfSize = (int32_t) 54 + (dib.biSizeImage);
}
/*}}}  */
/*{{{  write_dib(FILE *fp)*/
int
write_dib(fp)
    FILE *fp;
{
    if (1 != (fwrite(&dib.bfType, sizeof(dib.bfType), 1, fp))) return(1);
    if (1 != (fwrite(&dib.bfSize, sizeof(dib.bfSize), 1, fp))) return(1);
    if (1 != (fwrite(&dib.bfReserved1, sizeof(dib.bfReserved1), 1, fp))) return(1);
    if (1 != (fwrite(&dib.bfReserved2, sizeof(dib.bfReserved2), 1, fp))) return(1);
    if (1 != (fwrite(&dib.bfOffBits, sizeof(dib.bfOffBits), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biSize, sizeof(dib.biSize), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biWidth, sizeof(dib.biWidth), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biHight, sizeof(dib.biHight), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biPlanes, sizeof(dib.biPlanes), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biBitCount, sizeof(dib.biBitCount), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biCompression, sizeof(dib.biCompression), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biSizeImage, sizeof(dib.biSizeImage), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biXpelsPerMeter, sizeof(dib.biXpelsPerMeter), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biYpelsPerMeter, sizeof(dib.biYpelsPerMeter), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biClrUsed, sizeof(dib.biClrUsed), 1, fp))) return(1);
    if (1 != (fwrite(&dib.biClrImportant, sizeof(dib.biClrImportant), 1, fp))) return(1);
    return(0);
}
/*}}}  */
/*{{{  write_pixel(FILE *fp, color)*/
int
write_pixel(fp, color)
    FILE *fp;
    unsigned char color;
{
    unsigned char rgb[3];
    const int32_t color_table[C_MAX + 1] = {V_GREEN, V_RED, V_BLUE, V_WHITE, V_YELLOW, V_CYAN, V_PINK};
                                          /*    GND,   POS,    NEG,     AIR,      SUB,    FIL,    INS  */
    if (color > C_MAX) return 2;
    rgb[0] = (unsigned char)(0xff & (color_table[color]      ));
    rgb[1] = (unsigned char)(0xff & (color_table[color] >>  8));
    rgb[2] = (unsigned char)(0xff & (color_table[color] >> 16));
    if (1 != (fwrite(rgb, sizeof(rgb), 1, fp))) return(1);
    return(0);
}
/*}}}  */
/*{{{  draw_filled_circle(bmp, color, x0, y0, x1, y1);*/
/* Caution! use with care, as this function will not check the boundaries of 'bmp'. */
int
draw_filled_circle(bmp, color, x0, y0, x1, y1)
    unsigned char **bmp;
    unsigned char   color;
    int32_t         x0;
    int32_t         y0;
    int32_t         x1;
    int32_t         y1;
{
    int32_t         x, y, a, b, dd;
    int32_t         t;

    /*{{{  initialize variables*/
    t = gcd(abs(x1 - x0), abs(y1 - y0));
    a = (x1 - x0) / t;
    b = (y1 - y0) / t;
    dd = (x1 - x0) * (y1 - y0) * a * b;
    /*}}}  */

    if (x0 == x1) {
        /*{{{  draw vertical line*/
        for (y = min(y0, y1); y <= max(y0, y1); y++) {
            bmp[y][x0] = color;
        }
        /*}}}  */
    } else if (y0 == y1) {
        /*{{{  draw holizontal line*/
        for (x = min(x0, x1); x <= max(x0, x1); x++) {
            bmp[y0][x] = color;
        }
        /*}}}  */
    } else {
        for (y = min(y0, y1); y < max(y0, y1); y++) {
            for (x = min(x0, x1); x < max(x0, x1); x++) {
                /*{{{  Determine if it is inside an ellipse (in double scale)*/
                if (sqr(b * (2 * x + 1 - (x1 + x0))) + sqr(a * (2 * y + 1 - (y1 + y0))) < dd) {
                    bmp[y][x] = color;
                }
                /*}}}  */
                /*{{{  COMMENT Inequality for the interior of an ellipse*/
                /*b^2 (2x - x_1 - x_0 + 1)^2 + a^2 (2y - y_1 - y_0 + 1)^2  \leq  a b\ (x_1 - x_0) (y_1 - y_0)*/
                /*}}}  */
            }
        }
    }
    return 0;
}
/*}}}  */
/*{{{  draw_filled_square(bmp, color, x0, y0, x1, y1);*/
/* Caution! use with care, as this function will not check the boundaries of 'bmp'. */
int
draw_filled_square(bmp, color, x0, y0, x1, y1)
    unsigned char **bmp;
    unsigned char   color;
    int32_t         x0;
    int32_t         y0;
    int32_t         x1;
    int32_t         y1;
{
    int32_t         x, y;
    if ((x0 < 0) || (x1 < 0) || (y0 < 0) || (y1 < 0)) {
        return -1;
    } else {
        /*{{{  fill*/
        for (y = min(y0, y1); y < max(y0, y1); y++) {
            for (x = min(x0, x1); x < max(x0, x1); x++) {
                bmp[y][x] = color;
            }
        }
        /*}}}  */
        return 0;
    }
}
/*}}}  */
int
main(argc, argv)
    int argc;
    char *argv[];
{
    /*{{{  variables*/
    int             flags = 0;
    FILE           *outfile;
    double          t;    /* thickness */
    double          s;    /* spacing */
    double          w;    /* width */
    double          h;    /* height */
    double          d;    /* buring depth*/
    double          g;    /* signal gap */
    double          H;    /* box height */
    double          W;    /* box width */
    double          c;    /* coating thickness */
    double          er;   /* relative permitivity of substrate */
    double          ef;   /* relative permitivity of filler */
    double          ec;   /* relative permitivity of coating */
    unsigned char **bp   = NULL;
    unsigned char **bmp  = NULL;;
    size_t          memsize = 0;
    int             rtnv = 0;
    time_t          tloc;
    int             i, x, y, rim;
    int32_t         x_size, y_size;
    int             thickness, spacing, width, height, depth, gap, coating, box_height, box_width;
    int             side_ground_width;
    char           *c_comment[2] = {"/*", "*/"};
    char           *sh_comment[2] = {"#", ""};
    char           *soc;  /* start of comment */
    char           *eoc;  /* end of comment */
    char           *timestamp;
    char           *filename = NULL;
    /*}}}  */
    /*{{{  parse*/
    rtnv = parse(argc, argv, &flags, &filename, &w, &s, &h, &t, &d, &g, &c, &er, &ef, &ec, &H, &W);
    /*}}}  */
    if (!rtnv) {
        /*{{{  initialize variables*/
        thickness  = (int)round(fabs(t));
        spacing    = (int)round(fabs(s));
        width      = (int)round(fabs(w));
        height     = (int)round(fabs(h));
        depth      = (int)round(fabs(d));
        gap        = (int)round(fabs(g));
        coating    = (int)round(fabs(c));
        box_height = (int)round(fabs(H));
        box_width  = (int)round(fabs(W));
        /*{{{  adjust boxwidth*/
        if (flags & F_HINOMARU) {
            /*{{{  adjust proportion for hinomaru*/
            if (width % 12) {
                width += 12 - (width % 12);
                fprintf(stderr, "%s : The conductor width was adjusted to %d to be multiple of 12.\n", myname, width);
            }
            thickness  =  width;
            height     = (width / 12) *  4;
            box_height = (width / 12) * 20;
            box_width  = (width / 12) * 30;
            /*}}}  */
        } else if (flags & F_COUPLER) {
            if ((box_width - gap  ) & 1) {
                /*{{{  adjust (+1) box width*/
                box_width += 1;
                fprintf(stderr, "%s : The box width adjusted to %d\n", myname, box_width);
                /*}}}  */
            }
        } else {
            if ((box_width - width) & 1) {
                /*{{{  adjust (+1) box width*/
                box_width += 1;
                fprintf(stderr, "%s : The box width adjusted to %d\n", myname, box_width);
                /*}}}  */
            }
        }
        /*}}}  */
        /*{{{  adjust boxwidth for minimum width*/
        if (flags & F_COUPLER) {
            if (box_width < (2 * (spacing + width) + gap)) {
                /*{{{  adjust box width for the spacing and width*/
                box_width = 2 * (spacing + width) + gap;
                fprintf(stderr, "%s : The box width adjusted to %d\n", myname, box_width);
                /*}}}  */
            }
        } else {
            if (box_width < (2 * spacing + width)) {
                /*{{{  adjust box width for the spacing and width*/
                box_width = 2 * spacing + width;
                fprintf(stderr, "%s : The box width adjusted to %d\n", myname, box_width);
                /*}}}  */
            }
        }
        /*}}}  */
        /*{{{  calculate side ground width*/
        if (flags & F_COUPLER) {
            /*{{{  calculate side ground width*/
            if (flags & F_STRIP) {
                side_ground_width = ((box_width - (2 * width + gap)) / 2);
            } else {
                side_ground_width = ((box_width - gap) / 2) - (spacing + width);
            }
            /*}}}  */
        } else {
            /*{{{  calculate side ground width*/
            if (flags & F_STRIP) {
                side_ground_width = ((box_width - width) / 2);
            } else {
                side_ground_width = ((box_width - width) / 2) - spacing;
            }
            /*}}}  */
        }
        /*}}}  */
        rim        = (int)(RIM_SIZE);
        x_size     = box_width  + 2 * rim;
        y_size     = box_height + 2 * rim;
        memsize = (size_t)(y_size) * (size_t)(x_size);
        /*{{{  DEBUG check minimum dimensions*/
        if (flags & F_DEBUG) {
            if ( (thickness <= GRID_MIN)
               || (width <= GRID_MIN)
               || (height <= GRID_MIN)
               || (gap <= GRID_MIN)
               || (spacing <= GRID_MIN)) {
               errmsg(E_SMALL_DIM); /* show warnig and continue. */
               /* We don't care about the dimensions of the dielectrics here. */
            }
        }
        /*}}}  */
        /*{{{  set comment characters*/
        if(flags & F_ARRAY){
            soc = c_comment[0];
            eoc = c_comment[1];
        }else{
            soc = sh_comment[0];
            eoc = sh_comment[1];
        }
        /*}}}  */
        /*}}}  */
        /*{{{  open outout file*/
        if (NULL == filename) {
            return(errmsg(E_FILE));
        }
        if (NULL == (outfile = fopen(filename, "w"))){
            return(errmsg(E_FOPEN));
        }
        /*}}}  */
        /*{{{  allocate memory*/
        if (NULL == (bp =  (unsigned char **)malloc(y_size * sizeof(unsigned char *)))) {
            return(errmsg(E_MALLOC));
        }
        /*{{{  print DEBUG information*/
        if (flags & F_DEBUG) {
            fprintf(stderr, "%s %s : %d byte allocated.%s\n", soc, myname, y_size, eoc);
        }
        /*}}}  */
        for (i = 0; i < y_size; i++) {
            if (NULL == (bp[i] = (unsigned char *)malloc(x_size * sizeof(unsigned char)))) {
                return(errmsg(E_MALLOC));
            }
        }
        /*{{{  print DEBUG information*/
        if (flags & F_DEBUG) {
            fprintf(stderr, "%s %s : %d x %d byte allocated.%s\n", soc, myname, y_size, x_size, eoc);
        }
        /*}}}  */
        bmp = bp;
        /*}}}  */
        /*{{{  get timestamp*/
        time(&tloc);
        timestamp=ctime(&tloc);
        timestamp[strlen(timestamp) - 1] = '\0';
        /*}}}  */
        /*{{{  print header*/
        if (!(flags & F_QUIET)) {
            /*{{{  print program name*/
            if(flags & F_VERBOSE) {
                fprintf(stdout, "%s %s : %s %s %s\n",soc, myname, version, copyright, eoc);
            }
            /*}}}  */
            /*{{{  print property*/
            if (flags & F_CSV) {
                /*{{{  CSV*/
                /*{{{  print heading*/
                if(flags & F_VERBOSE) {
                    if (flags & F_PTIME) {
                        /*{{{  print heading with timestamp*/
                        if (flags & F_COUPLER) {
                            fprintf(stdout, "filename,w,g,s,h,t,d,er,ef,W,H,timestamp\n");
                        } else {
                            if (flags & F_WIRE) {
                                fprintf(stdout, "filename,w,s,h,t,d,c,er,ef,ec,W,H,timestamp\n");
                            } else {
                                fprintf(stdout, "filename,w,s,h,t,d,er,ef,W,H,timestamp\n");
                            }
                        }
                        /*}}}  */
                    } else {
                        /*{{{  print heading without timestamp*/
                        if (flags & F_COUPLER) {
                            fprintf(stdout, "filename,w,g,s,h,t,d,er,ef,W,H\n");
                        } else {
                            if (flags & F_WIRE) {
                                fprintf(stdout, "filename,w,s,h,t,d,c,er,ef,ec,W,H\n");
                            } else {
                                fprintf(stdout, "filename,w,s,h,t,d,er,ef,W,H\n");
                            }
                        }
                        /*}}}  */
                    }
                }
                /*}}}  */
                /*{{{  print properties*/
                if (flags & F_PTIME) {
                    /*{{{  print properties with timestamp*/
                    if (flags & F_WIRE) {
                        fprintf(stdout, "%s,%d,%d,%d,%d,%d,%d,%g,%g,%g,%d,%d,'%s'\n", filename,width, spacing, height, thickness, depth, coating, er, ef, ec, box_width, box_height, timestamp);
                    } else {
                        fprintf(stdout, "%s,%d,%d,%d,%d,%d,%g,%g,%d,%d,'%s'\n", filename, width, spacing, height, thickness, depth, er, ef, box_width, box_height, timestamp);
                    }
                    /*}}}  */
                } else {
                    /*{{{  print properties without timestamp*/
                    if (flags & F_WIRE) {
                        fprintf(stdout, "%s,%d,%d,%d,%d,%d,%d,%g,%g,%g,%d,%d\n", filename,width, spacing, height, thickness, depth, coating, er, ef, ec, box_width, box_height);
                    } else {
                        fprintf(stdout, "%s,%d,%d,%d,%d,%d,%g,%g,%d,%d\n", filename,width, spacing, height, thickness, depth, er, ef, box_width, box_height);
                    }
                    /*}}}  */
                }
                /*}}}  */
                /*}}}  */
            } else if (flags & F_ARRAY ) {
                /*{{{  array*/
                if (flags & F_PTIME) {
                    /*{{{  print timestamp*/
                    fprintf(stdout, "%s %s %s\n", soc, timestamp, eoc);
                    /*}}}  */
                }
                if(flags & F_VERBOSE) {
                    /*{{{  print heading*/
                    if (flags & F_COUPLER) {
                        fprintf(stdout, "%s %s, w, g, s, h, t, d, er, ef, W, H, %s\n", soc, myname, eoc);
                    } else {
                        if (flags & F_WIRE) {
                            fprintf(stdout, "%s %s, w, s, h, t, d, c, er, ef, ec, W, H %s\n", soc, myname, eoc);
                        } else {
                            fprintf(stdout, "%s %s, w, s, h, t, d, er, ef, W, H %s\n", soc, myname, eoc);
                        }
                    }
                    /*}}}  */
                }
                /*{{{  print properties*/
                if (flags & F_COUPLER) {
                    fprintf(stdout, "{%d, %d, %d, %d, %d, %d, %g, %g, %d, %d} %s %s %s\n",
                        width, gap, spacing, height, thickness, depth, er, ef, box_width, box_height, soc, filename, eoc);
                } else {
                    if (flags & F_WIRE) {
                        fprintf(stdout, "{%d, %d, %d, %d, %d, %d, %g, %g, %g, %d, %d} %s %s %s\n",
                            width, spacing, height, thickness, depth, coating, er, ef, ec, box_width, box_height, soc, filename, eoc);
                    } else {
                        fprintf(stdout, "{%d, %d, %d, %d, %d, %g, %g, %d, %d} %s %s %s\n",
                            width, spacing, height, thickness, depth, er, ef, box_width, box_height, soc, filename, eoc);
                    }
                }
                /*}}}  */
                /*}}}  */
            } else {
                /*{{{  default*/
                /*{{{  print timestamp*/
                if (flags & F_PTIME) {
                    fprintf(stdout, "%s %s %s\n", soc, timestamp, eoc);
                }
                /*}}}  */
                /*{{{  print property*/
                if (flags & F_COUPLER) {
                    fprintf(stdout, "%s %s : w=%d g=%d s=%d h=%d t=%d d=%d er=%g ef=%g W=%d H=%d %s\n",
                        soc, filename, width, gap, spacing, height, thickness, depth, er, ef, box_width, box_height, eoc);
                } else {
                    if (flags & F_WIRE) {
                        fprintf(stdout, "%s %s : w=%d s=%d h=%d t=%d d=%d c=%d er=%g ef=%g ec=%g W=%d H=%d %s\n",
                            soc, filename, width, spacing, height, thickness, depth, coating, er, ef, ec, box_width, box_height, eoc);
                    } else {
                        fprintf(stdout, "%s %s : w=%d s=%d h=%d t=%d d=%d er=%g ef=%g W=%d H=%d %s\n",
                            soc, filename, width, spacing, height, thickness, depth, er, ef, box_width, box_height, eoc);
                    }
                }
                /*}}}  */
                /*}}}  */
            }
            /*}}}  */
        } else if (flags & F_ATLC) {
            /*{{{  print property to stderr*/
            if (flags & F_COUPLER) {
                fprintf(stderr, "%s %s : w=%d g=%d s=%d h=%d t=%d d=%d er=%g ef=%g W=%d H=%d %s\n",
                    soc, filename, width, gap, spacing, height, thickness, depth, er, ef, box_width, box_height, eoc);
            } else {
                if (flags & F_WIRE) {
                    fprintf(stderr, "%s %s : w=%d s=%d h=%d t=%d d=%d c=%d er=%g ef=%g ec=%g W=%d H=%d %s\n",
                        soc, filename, width, spacing, height, thickness, depth, coating, er, ef, ec, box_width, box_height, eoc);
                } else {
                    fprintf(stderr, "%s %s : w=%d s=%d h=%d t=%d d=%d er=%g ef=%g W=%d H=%d %s\n",
                        soc, filename, width, spacing, height, thickness, depth, er, ef, box_width, box_height, eoc);
                }
            }
            /*}}}  */
        }
        /*}}}  */
        /*{{{  draw structure*/
        /*{{{  fill gnd*/
        for (y = 0; y < y_size; y++) {
            for (x = 0; x < x_size; x++) {
                bmp[y][x] = C_GND;
            }
        }
        /*}}}  */
        /*{{{  draw air*/
        draw_filled_square(bmp, C_AIR, rim, rim, box_width + rim, box_height + rim);
        /*}}}  */
        /*{{{  draw substraight and filler*/
        draw_filled_square(bmp, er == 1.0 ? C_AIR : C_SUB, rim, rim, rim + box_width, rim + height);
        draw_filled_square(bmp, er == 1.0 ? C_AIR : er == ef ? C_SUB : C_FIL, rim, rim + height, rim + box_width, rim + height + depth);
        /*}}}  */
        /*{{{  draw ground conductors*/
        /* thickness, spacing, width, height, depth, gap, box_height, box_width; */
        if (!(flags & F_STRIP )) {
            /*{{{  draw coplanar ground*/
            draw_filled_square(bmp, C_GND, rim         , rim + height,
                                           rim + side_ground_width, rim + height + thickness);
            draw_filled_square(bmp, C_GND, rim + box_width - side_ground_width, rim + height,
                                           rim + box_width         , rim + height + thickness );
            /*}}}  */
        }
        /*{{{  print DEBUG information*/
        if (flags & F_DEBUG) {
            fprintf(stderr, "%s %s : side_ground_width=%d, box_width=%d%s\n",soc, myname, side_ground_width, box_width, eoc);
        }
        /*}}}  */
        /*}}}  */
        if (flags & F_HINOMARU) {
            /*{{{  draw signal+ conductor*/
            draw_filled_circle(bmp, C_POS, rim + side_ground_width + spacing        , rim + height,
                                           rim + side_ground_width + spacing + width, rim + height + thickness);
            /*}}}  */
        } else if (flags & F_COUPLER) {
            if (flags & F_WIRE) {
                /*{{{  draw +wire coating*/
                draw_filled_circle(bmp, C_INS, rim + side_ground_width + spacing         - coating, rim + height,
                                               rim + side_ground_width + spacing + width + coating, rim + height + width + 2 * coating);
                /*}}}  */
                /*{{{  draw -wire coating*/
                draw_filled_circle(bmp, C_INS, rim + side_ground_width + spacing + width + gap - coating        , rim + height,
                                               rim + side_ground_width + spacing + width + gap + coating + width, rim + height + width + 2 * coating);
                /*}}}  */
                /*{{{  draw +wire conductor*/
                draw_filled_circle(bmp, C_POS, rim + side_ground_width + spacing        , rim + height + coating,
                                               rim + side_ground_width + spacing + width, rim + height + coating + width);
                /*}}}  */
                /*{{{  draw -wire conductor*/
                draw_filled_circle(bmp, C_NEG, rim + side_ground_width + spacing + width + gap        , rim + height + coating,
                                               rim + side_ground_width + spacing + width + gap + width, rim + height + coating + width);
                /*}}}  */
            } else {
                /*{{{  draw signal+ conductors*/
                draw_filled_square(bmp, C_POS, rim + side_ground_width + spacing        , rim + height,
                                               rim + side_ground_width + spacing + width, rim + height + thickness);
                /*}}}  */
                /*{{{  draw signal- conductors*/
                draw_filled_square(bmp, C_NEG, rim + side_ground_width + spacing + width + gap        , rim + height,
                                               rim + side_ground_width + spacing + width + gap + width, rim + height + thickness);
                /*}}}  */
            }
        } else if (flags & F_WIRE) {
            /*{{{  draw coating*/
            draw_filled_circle(bmp, C_INS, rim + side_ground_width + spacing         - coating, rim + height,
                                           rim + side_ground_width + spacing + width + coating, rim + height + width + 2 * coating);
            /*}}}  */
            /*{{{  draw signal+ conductor*/
            draw_filled_circle(bmp, C_POS, rim + side_ground_width + spacing         , rim + height + coating,
                                           rim + side_ground_width + spacing + width , rim + height + coating + width);
            /*}}}  */
        } else {
            /*{{{  draw signal+ conductor*/
            draw_filled_square(bmp, C_POS, rim + side_ground_width + spacing        , rim + height,
                                           rim + side_ground_width + spacing + width, rim + height + thickness);
            /*}}}  */
        }
        /*}}}  */
        /*{{{  init dib*/
        /*{{{  print DEBUG information*/
        if (flags & F_DEBUG) {
            fprintf(stderr, "%s %s : x_size=%d, y_size=%d %s\n",soc, myname, x_size, y_size, eoc);
        }
        /*}}}  */
        init_dib(x_size, y_size);
        /*}}}  */
        /*{{{  write dib header*/
        write_dib(outfile);
        /*}}}  */
        /*{{{  write bitmap*/
        for (y = 0 ; y < box_height + 2 * rim; y++) {
            for (x = 0 ; x < box_width + 2 * rim; x++) {
                if(write_pixel(outfile, bmp[y][x])) {
                    exit(errmsg(E_FWRITE));
                };
            }
            /*{{{  padding*/
            for (i = 3 * x ; (i % 4) != 0; i++) {
                if (EOF == fputc(0, outfile)) exit(errmsg(E_FWRITE));
            }
            /*}}}  */
        }
        /*{{{  print DEBUG information*/
        if (flags & F_DEBUG) {
            fprintf(stderr, "%s %s : %d-pixels x %d-lines written.%s\n",soc, myname, box_width + 2 * rim, box_height + 2, eoc);
        }
        /*}}}  */
        /*}}}  */
        /*{{{  free memory*/
        for (i = 0; i < y_size; i++) {
            free(bp[i]);
        }
        free(bp);
        /*}}}  */
        /*{{{  close output file*/
        fclose(outfile);
        /*}}}  */
        /*{{{  print command line for atlc*/
        if (flags & F_ATLC) {
            fprintf(stdout, "atlc -d 0x%06x=%g -d 0x%06x=%g -d 0x%06x=%g %s\n", V_YELLOW, er, V_CYAN, ef, V_PINK, ec, filename);
        }
        /*}}}  */
    }
    exit(errmsg(rtnv));
}
